<?php

function create_tmp_dir()
{
	global $options;

	if (!file_exists($options['tmpdir'])) {
		if (!mkdir($options['tmpdir'], 0755))
			throw new Mirror_Exception("Unable to create the tmp directory '" . $options['tmpdir'] . "'");
	} else {
		if (!is_dir($options['tmpdir']))
			throw new Mirror_Exception("Path exists but it is not directory '" . $options['tmpdir'] . "'");
	}
}

function do_exec($cmd)
{
        $list = array();
        $ret  = 0;

        exec($cmd, $list, $ret);

        if ($ret !== 0) {
                $size = sizeof($list);
                $msg = (0 < $size) ? implode("\n", $list) : "Empty error message from exec command: $cmd\n";
		throw new Mirror_Exception($msg);
        }

        return $list;
}

function get_config_content($config, $repos = null, $dst_config = null)
{
	global $options;

	if (!$config)
		throw new Mirror_Exception("Config is not defined");

	if (!$dst_config)
		$dst_config = $config;

	if (!$repos)
		$repos = $options['repos'];

	echo_try("get '$config' from '" . $repos . "'");
	$cmd = $options['rmcmd'] . " -f " . $options['tmpdir'] . "/$config;";
	$cmd .= "mkdir -p " . $options['tmpdir'] . "/`dirname $config`;";
	$cmd .= 'rsync -auv --copy-links ' . $repos . "/$config " . $options['tmpdir'] . "/$dst_config";
	do_exec($cmd);
	echo_ok();

	$content = file_get_contents($options['tmpdir'] . "/" . $dst_config);
	if (!$content)
		throw new Mirror_Exception("Unable to get data from config: $dst_config");

	return $content;
}

function save_config($file, $xml)
{
	$dom = new DOMDocument('1.0');
	$dom->preserveWhiteSpace = false;
	$dom->formatOutput = true;
	$dom->loadXML($xml->asXML());

	// Strip comments
	$xp = new DOMXpath($dom);
	$nodes = $xp->query('//comment()');
	for ($i = 0; $i < $nodes->length; $i++)
		$nodes->item($i)->parentNode->removeChild($nodes->item($i));

	$dom->save($file);
}

function load_config($file)
{
	$content = get_config_content($file);
	$xml = new SimpleXMLElement($content);
	return $xml;
}

function print_version($progname)
{
	global $options;
	echo $progname . " version => '" . $options['version'] . "'\n";
}

function echo_try($str)
{
	echo "Trying to " . $str . "... ";
}

function echo_ok()
{
	echo "ok\n";
}

function echo_skip($str = null)
{
	if (!str) {
		echo "skip\n";
		return;
	}
	echo "$str\n";
}

function join_paths() {
	$paths = array();

	foreach (func_get_args() as $path) {
		if ($path !== '' && $path !== '.')
			$paths[] = $path;
	}

	return join('/', $paths);
}
