<?php

class Product_Addons extends Product_Common
{
	public function __construct()
	{
		$this->data = $this->get_data();
		return $this;
	}

	protected function get_data_info($release, $data, $basedir)
	{
		$attr = $release->attributes();

		if ($attr->version) {
			$id = (string)$attr->id;

			$data[$id]['data'] = array();
			$data[$id]['info']['name'] = (string)$attr->name;
			$data[$id]['info']['version'] = (string)$attr->version;

			// This addon works for follow versions
			$data[$id]['info']['from_version'] = '';
			$data[$id]['info']['to_version'] = '';

			if ($release->compatibility_info) {
				foreach ($release->compatibility_info->compatible as $entry) {
					$battr = $entry->attributes();
					if ((string)$battr->product_id == "plesk" || (string)$battr->product_id == "panel") {
						$data[$id]['info']['from_version'] = (string)$battr->from_version;
						$data[$id]['info']['to_version']   = (string)$battr->to_version;
					}
				}
			}

			// Gathering info by oses
			foreach ($release->build AS $build) {
				if (!isset($data[$id]['info']['directory']))
					$data[$id]['info']['directory'] = join_paths($basedir, dirname((string)$build->attributes()->config));

				$name = $this->get_name($build);
				if (!isset($name) || empty($name))
					continue;

				$type = $this->get_type($build);
				$battr = (array)$build->attributes();

				$data[$id]['data'][$name] = $battr['@attributes'];
				$data[$id]['data'][$name]['type'] = $type;
			}
		}
		return $data;
	}

	protected function get_cfglist()
	{
		$xmlObj = load_config('products.inf3');

		$products = $xmlObj->vendor->products;
		foreach (array($products->addon, $products->lazy_addon) AS $addons) {
			foreach ($addons AS $addon) {
				$attr = $addon->attributes();
				$conf[] = array(
					'id'        => (string)$attr->id,
					'name'      => (string)$attr->name,
					'reference' => (string)$attr->reference
				);
			}
		}
		return $conf;
	}

	public function get_info(Config $config)
	{
		$releases = $this->get_release_list();
		$cfg_releases = $config->get_release_list();

		// iterate by addon releases
		foreach ($releases AS $rel) {
			$ver_from = $this->data[$rel]['info']['from_version'];
			$ver_to   = $this->data[$rel]['info']['to_version'];
			// iterate by requred releases
			foreach ($cfg_releases AS $cfg_rel) {
				$prodver = (string)$config->get_release_version($cfg_rel);
				if (!$prodver)
					throw new Mirror_Config_Exception("Unable to find product version");

				$match = ((version_compare($ver_from, $prodver) <= 0) &&
					(version_compare($ver_to, $prodver) >= 0));
				if ($match) {
					$os = $this->get_os_list($rel);
					$params['addons'][$rel]['info'] = $this->data[$rel]['info'];
					foreach ($os AS $key)
						$params['addons'][$rel]['data'][$key] = $this->data[$rel]['data'][$$key];
				}
			}
		}
		return $params;
	}

	public function get_cmd($data, $cmd_url, $path)
	{
		$opts = getCmdOptions();
		if ($opts->is_win())
			return $this->get_cmd_win($data, $cmd_url, $path);

		return $this->get_cmd_unix($data, $cmd_url, $path);
	}

	private function get_cmd_win($data, $cmd_url, $path)
	{
		global $options;

		$cmds = array();
		$cmd_prefix = "copy_target " . $cmd_url;

		foreach ($data['addons'] AS $key => $val) {
			$src_prefix = $cmd_prefix . "/" . $val['info']['directory'];
			$dst_prefix = $path . "/";

			$cmd['descr'] = "Create the directory for Addons";
			$cmd['param'] = "mkdir -p $dst_prefix/";
			$cmds[] = $cmd;

			$cmd['descr'] = "Getting the addon component $key";
			$cmd['param'] = $src_prefix . " " . $dst_prefix;
			$cmds[] = $cmd;
		}

		return $cmds;
	}

	private function get_cmd_unix($data, $cmd_url, $path)
	{
		global $options;

		$cmds = array();
		$cmd_prefix = "copy_target " . $cmd_url;

		foreach ($data['addons'] AS $key => $val) {
            if (dirname($val['info']['directory']) != '.') {
            // EOL'ed PHP with referenced to exact build config="pool/PHP_7.1.33_96/php71-cos6-x86_64.inf3"
                $base_dir = dirname($val['info']['directory']);

			    $cmd['descr'] = "Create the directory for Addons";
			    $cmd['param'] = "mkdir -p $path/$base_dir";
			    $cmds[] = $cmd;

    			$cmd['descr'] = "Getting the addon component $key";
	    		$cmd['param'] = $cmd_prefix . "/" . $val['info']['directory'] . " " . $path . "/" . $base_dir;
		    	$cmds[] = $cmd;
            } else {

                $src_prefix = $cmd_prefix . "/" . $val['info']['directory'];
                $dst_prefix = $path . "/";

                $cmd['descr'] = "Create the directory for Addons";
                $cmd['param'] = "mkdir -p $dst_prefix/";
                $cmds[] = $cmd;

                $cmd['descr'] = "Getting the addon component $key";
                $cmd['param'] = $src_prefix . " " . $dst_prefix;
                $cmds[] = $cmd;

                // Debian repos
                $dir = $path . "/debian/";
                $str = $cmd_prefix . "/debian/" . $val['info']['directory'] . " " . $dir . "/";

                $cmd['descr'] = "Create the directory for Debian apt cache";
                $cmd['param'] = "mkdir -p $dir/";
                $cmds[] = $cmd;

                $cmd['descr'] = "Create Debian apt cache";
                $cmd['param'] = $str . " 1>/dev/null 2>&1";
                $cmds[] = $cmd;

                // Ubuntu repos
                $dir = $path . "/ubuntu/";
                $str = $cmd_prefix . "/ubuntu/" . $val['info']['directory'] . " " . $dir . "/";

                $cmd['descr'] = "Create the directory for Ubuntu apt cache";
                $cmd['param'] = "mkdir -p $dir/";
                $cmds[] = $cmd;

                $cmd['descr'] = "Create Ubuntu apt cache";
                $cmd['param'] = $str . " 1>/dev/null 2>&1";
                $cmds[] = $cmd;
            }
		}

		return $cmds;
	}
}