<?php

class Product_Common implements Product_Interface
{
	var $data = array();

        public function get_data()
        {
		foreach ($this->get_cfglist() AS $cfg) {
			$xml = load_config($cfg['reference']);
			foreach (array($xml->release, $xml->release_1) AS $product) {
				foreach ($product AS $release)
					$data = $this->get_data_info($release, $data, dirname($cfg['reference']));
			}
		}
		return $data;
        }

	public function get_release_list()
	{
		$rels = array();

		foreach ($this->data AS $key => $val)
			if (!empty($val['data']))
				$rels[] = $key;

		return $rels;
	}

	public function get_os_list($release)
	{
		if (!$this->data[$release])
			throw new Mirror_Exception("Unknown release '$release'");

		$os = array();
		foreach ($this->data[$release]['data'] AS $key => $val)
			$os[] = $key;

		return $os;
	}

	public function get_name($entry)
	{
		$convert = array("Debian" => "deb",
				"Ubuntu"  => "ubt",
				"SuSE"    => "suse",
				"FreeBSD" => "fr",
				"CentOS"  => "cos",
				"RedHat"  => "rh",
				"AlmaLinux" => "al",
				"Rocky" => "ro",
				"FedoraCore" => "fc",
				"CloudLinux" => "cl",
				"Microsoft"  => "win",
		);

		$attr = $entry->attributes();

		// Checking for unknown operating systems
		if (!isset($convert[(string)$attr->os_vendor]))
			return;

		$name = $convert[(string)$attr->os_vendor] . (string)$attr->os_version . '_' . (string)$attr->os_arch;
		return $name;
	}

	protected function get_type($build)
	{
		$str = (string)$build->attributes()->os_vendor;
		if (($str == "Ubuntu") || ($str == "Debian"))
			return "deb";

		if ($str == "Microsoft")
			return "msi";

		return "rpm";
	}

	protected function get_cfglist() {
		$conf[] = array(
			'id'        => 'products',
			'name'      => 'Plesk Panel Products',
			'reference' => 'products.inf3'
		);
		return $conf;
	}

	public function get_info(Config $config)
	{
		return array();
	}

	public function get_cmd($data, $cmd_url, $path)
	{
		global $options;

		$cmd_prefix = "copy_target " . $cmd_url;

		foreach ($data['configs'] AS $key => $val) {
			$cmds[$key]['descr'] = "Sync config for " . $val['name'];
			$cmds[$key]['param'] = $cmd_prefix . "/" . $val['reference'] . " " . $path . "/";
		}

		$tmp = array(
			'descr' => 'Parallels Installer directory',
			'param' => $cmd_prefix . "/Parallels_Installer" . " " . $path . "/"
		);
		$cmds[] = $tmp;

		return $cmds;
	}

	protected function strip_config(Config $config, $file, $file_new)
	{
		$content = get_config_content($file);
		file_put_contents($file_new, $content);
	}

	protected function merge_config($path, Config $config, $file, $file_new)
	{
		$content = get_config_content($file);
		file_put_contents($file_new, $content);
	}

	public function upload_config($type, Config $config)
	{
		global $options;

		$opts = getCmdOptions();
		$path = $opts->get_path();

		$action = Action::factory();
		foreach ($this->get_cfglist() AS $cfg) {
			$file = $cfg['reference'];

			if ($type == Types::STRIP) {
				$dstfile = $options['tmpdir'] . "/" . $file . "_update";
				$this->strip_config($config, $file, $dstfile);
			} else {
				$dstfile = $options['tmpdir'] . "/" . $file . "_push";
				$this->merge_config($path, $config, $file, $dstfile);
			}
			$action->upload($dstfile, $path . "/" . $file, 0644);
		}
	}
}

