<?php

require_once("lib/Common.php");

class Product_Plesk extends Product_Common
{
	var $data = array();

	function __construct()
	{
		$this->data = $this->get_data();
		return $this;
	}

	protected function get_data_info($release, $data, $basedir)
	{
		$attr = $release->attributes();
		if ($attr->version) {
			$id = (string)$attr->id;
			$data[$id]['data'] = array();
			$data[$id]['info']['name'] = (string)$attr->name;
			$data[$id]['info']['version'] = (string)$attr->version;
			$data[$id]['info']['details_url'] = (string)$attr->details_url;

			if ($release->build) {
				foreach ($release->build as $build) {
					if (!isset($data[$id]['info']['directory']))
						$data[$id]['info']['directory'] = join_paths($basedir, dirname((string)$build->attributes()->config));

					$name = $this->get_name($build);
					if (!isset($name) || empty($name))
						continue;

					$type = $this->get_type($build);
                                        $battr = (array)$build->attributes();

					$data[$id]['data'][$name] = $battr['@attributes'];
					$data[$id]['data'][$name]['type'] = $type;
				}
			}
		}
		return $data;
	}

	protected function get_cfglist()
	{
		$xmlObj = load_config('products.inf3');
		$products = $xmlObj->vendor->products;
		foreach ($products->product AS $product) {
			$attr = $product->attributes();
			$conf[] = array(
				'id'        => (string)$attr->id,
				'name'      => (string)$attr->name,
				'reference' => (string)$attr->reference
			);
		}
		return $conf;
	}

	public function get_info(Config $config)
	{
		// Gathering info for described in local config oses only
		$data = $this->get_data();
		$rels = $config->get_release_list();
		for ($i = 0; $rels[$i]; $i++) {
			$os = $config->get_os_list($rels[$i]);
			$params[$rels[$i]]['info'] = $data[$rels[$i]]['info'];
			for ($j = 0; $os[$j]; $j++)
				$params[$rels[$i]]['data'][$os[$j]] = $data[$rels[$i]]['data'][$os[$j]];
		}

		return $params;
	}

	public function get_cmd($data, $cmd_url, $path)
	{
		$opts = getCmdOptions();
		if ($opts->is_win())
			return $this->get_cmd_win($data, $cmd_url, $path);

		return $this->get_cmd_unix($data, $cmd_url, $path);
	}

	private function get_cmd_win($data, $cmd_url, $path)
	{
		global $options;

		$cmd_prefix = "copy_target " . $cmd_url;

		$cmd['descr'] = "Sync distributives of Autoinstaller";
		$cmd['param'] = $cmd_prefix . "/Autoinstaller_* " . $path . "/";
		$cmds[] = $cmd;

		foreach ($data AS $key => $val) {
			if (($key == 'addons'))
				continue;

			$src_prefix = $cmd_prefix . "/" . $val['info']['directory'] . "/";
			$dst_prefix = $path . "/" . $val['info']['directory'] . "/";

			// Create the root directory
			$cmd['descr'] = "Create the directory for $key";
			$cmd['param'] = "mkdir -p $dst_prefix";
			$cmds[] = $cmd;

			$cmd['descr'] = "Sync Plesk distributive  directory for $key";
			$cmd['param'] = $src_prefix . " " . $dst_prefix;
			$cmds[] = $cmd;
		}

		return $cmds;
	}

	private function get_cmd_unix($data, $cmd_url, $path)
	{
		global $options;

		$cmd_prefix = "copy_target " . $cmd_url;

		// Getting commands for components
		foreach ($data AS $key => $val) {
			if (($key == 'addons'))
				continue;

			$src_prefix = $cmd_prefix . "/" . $val['info']['directory'] . "/";
			$dst_prefix = $path . "/" . $val['info']['directory'] . "/";

			// FIXME: In the common way we should get
			// 'apt_repository' data from per-build config.
			$apt_prefix = $val['info']['directory'];
			if (preg_match("/SiteBuilder/", $dst_prefix))
				$apt_prefix = basename($val['info']['directory']);

			// Create the root directory
			$cmd['descr'] = "Create the directory for $key";
			$cmd['param'] = "mkdir -p $dst_prefix";
			$cmds[] = $cmd;

			$cmd['descr'] = "Getting extensions";
			$cmd['param'] = $cmd_prefix . "/extensions " . $path . "/";
			$cmds[] = $cmd;

			$cmd['descr'] = "Sync the microupdates directory for $key";
			$cmd['param'] = $src_prefix . "/microupdates" . " " . $dst_prefix . " 1>/dev/null 2>&1";
			$cmds[] = $cmd;

			$cmd['descr'] = "Sync the examiners directory for $key";
			$cmd['param'] = $src_prefix . "/examiners" . " " . $dst_prefix . " 1>/dev/null 2>&1";
			$cmds[] = $cmd;

			$cmd['descr'] = "Sync the dists directory for $key";
			$cmd['param'] = $src_prefix . "/dists" . " " . $dst_prefix . " 1>/dev/null 2>&1";
			$cmds[] = $cmd;

			$cmd['descr'] = "Getting plesk_getkeyinfo.sh";
			$cmd['param'] = $src_prefix . "/plesk_getkeyinfo.sh " . $dst_prefix . " 1>/dev/null 2>&1";
			$cmds[] = $cmd;

			$cmd['descr'] = "Getting Parallels_Installer";
			$cmd['param'] = $cmd_prefix . "/Parallels_Installer " . $path . "/";
			$cmds[] = $cmd;

			$cmd['descr'] = "Getting Installer";
			$cmd['param'] = $cmd_prefix . "/Installer " . $path . "/";
			$cmds[] = $cmd;

			foreach ($val['data'] AS $opt => $info) {
				$os = $info['type'] . "-" . $info['os_vendor'] . "-" . $info['os_version'] . "-" . $info['os_arch'];

				$cmd['descr'] = "Getting Plesk config for $key/$opt";
				$cmd['param'] = $src_prefix . "/" . basename($info['config']) . " " . $dst_prefix;
				$cmds[] = $cmd;

				if ($info['patches']) {
					$cmd['descr'] = "Getting Plesk patch config for $key/$opt";
					$cmd['param'] = $src_prefix . "/" . basename($info['patches']) . " " . $dst_prefix;
					$cmds[] = $cmd;
				}

				$cmd['descr'] = "Getting Plesk packages for $key/$opt";
				$cmd['param'] = $src_prefix . "dist-" . $os . " " . $dst_prefix;
				$cmds[] = $cmd;

				$cmd['descr'] = "Getting Plesk updates for $key/$opt";
				$cmd['param'] = $src_prefix . "update-" . $os . " " . $dst_prefix;
				$cmds[] = $cmd;

				$cmd['descr'] = "Getting Plesk thirdparty components for $key/$opt";
				$cmd['param'] = $src_prefix . "thirdparty-" . $os . " " . $dst_prefix;
				$cmds[] = $cmd;

				$cmd['descr'] = "Getting Plesk extra packages for $key/$opt";
				$cmd['param'] = $src_prefix . "extras-" . $os . " " . $dst_prefix;
				$cmds[] = $cmd;

				if ($info['os_vendor'] == "Debian") {
					$dir = $path . "/debian/";
					$str = $cmd_prefix . "/debian/" . $apt_prefix . " " . $dir . "/";
					$cmd['descr'] = "Create the directory for Debian apt cache";
					$cmd['param'] = "mkdir -p $dir/";
					$cmds[] = $cmd;

					$cmd['descr'] = "Create Debian apt cache";
					$cmd['param'] = $str;
					$cmds[] = $cmd;
				}

				if ($info['os_vendor'] == "Ubuntu") {
					$dir = $path . "/ubuntu/";
					$str = $cmd_prefix . "/ubuntu/" . $apt_prefix . " " . $dir . "/";
					$cmd['descr'] = "Create the directory for Ubuntu apt cache";
					$cmd['param'] = "mkdir -p $dir/";
					$cmds[] = $cmd;

					$cmd['descr'] = "Create Ubuntu apt cache";
					$cmd['param'] = $str . " 1>/dev/null 2>&1";
					$cmds[] = $cmd;
				}
			}
		}
		return $cmds;
	}

	protected function strip_config(Config $config, $file, $file_new)
	{
		$xml = load_config($file);
		$rel = $config->get_release_list();

		// We should remove an element just only
		// the next iteration or the cycle breaks otherwise
		foreach(array($xml->release, $xml->release_1) as $releases) {
			// Iterate by source releases
			foreach($releases as $release) {
				// We shouldn't use foreach for non existent 'build' element
				// Because it'll add one if doesn't exists. :-/
				if (!$release->build)
					continue;

				// Iterate by source OSes for release
				$remBuild = '';
				foreach ($release->build AS $build) {
					if ($remBuild) {
						$remBuild->parentNode->removeChild($remBuild);
						$remBuild = '';
					}
					// Iterate by customized releases
					$flag = true;
					foreach ($rel AS $name) {
						if ($name == $release->attributes()->id) {
							// This release is exists in custom list
							// So check it accurately and strip unwanted builds
							$oslist = $config->get_os_list($name);
							foreach ($oslist AS $os) {
								$osname = $this->get_name($build);
								if ($osname == $os) {
									$flag = false;
									break;
								}
							}
						}
					}
					if ($flag)
						$remBuild = dom_import_simplexml($build);
				}
				if ($remBuild)
					$remBuild->parentNode->removeChild($remBuild);
			}
		}

		save_config($file_new, $xml);
	}

	protected function merge_config($path, Config $config, $file, $file_new)
	{
		global $options;

		// Getting config from source repository
		// And merge only required releases/builds
		$tmpfile = $options['tmpdir'] . "/" . $file . "_strip_push";
		$this->strip_config($config, $file, $tmpfile);
		$srcxml = simplexml_load_file($tmpfile);

		$action = Action::factory();

		// FIXME: Hack. catch exception if file doesn't exists
		// But possibly an error will be recognized as the not existing file
		try {
			$action->download($path . "/$file", $options['tmpdir'] . "/" . $file . "_dst", 0644);
		} catch (Mirror_Exception $e) {
			// file not found
			save_config($file_new, $srcxml);
			return;
		}

		$dstxml = simplexml_load_file($options['tmpdir'] . "/" . $file . "_dst");

		foreach (array($srcxml->release, $srcxml->release_1) AS $rels) {
			foreach ($rels AS $src_release) {
				$rsrc_id = (string)$src_release->attributes()->id;

				// Add unknown release into destination config
				$flag = false;
				foreach (array($dstxml->release, $dstxml->release_1) AS $dst_rels) {
					foreach ($dst_rels AS $dst_release) {
						$rdst_id = (string)$dst_release->attributes()->id;
						// Iterate by known release to merge OSes
						if ($rsrc_id == $rdst_id) {
							$flag = true;

							// We shouldn't use foreach for non existent 'build' element
							// Because it'll add one if doesn't exists. :-/
							if (!$src_release->build)
								break;

							foreach($src_release->build AS $src_build) {
								$bflag = false;

								// Add build if does not exists
								if (!$dst_release->build) {
									$this->mergeXML($dst_release, $src_build);
									continue;
								}

								// Check attributes and fix them.
								$bs_attr = $src_build->attributes();
								$bs_str  = $bs_attr->os_vendor . $bs_attr->os_version . $bs_attr->os_arch;
								foreach ($dst_release->build AS $dst_build) {
									$bd_attr = $dst_build->attributes();
									$bd_str  = $bd_attr->os_vendor . $bd_attr->os_version . $bd_attr->os_arch;
									if ($bs_str == $bd_str) {
										$bflag = true;
										break;
									}
								}
								if ($bflag)
									$this->mergeXMLattr($dst_build, $src_build);
							}
							break;
						}
					}
				}
				// Release does not exists in config yet. Add them.
				if (!$flag)
					$this->mergeXML($dstxml, $src_release);
			}
		}

		save_config($file_new, $dstxml);
	}

	private function mergeXML(&$base, $add)
	{
		$new = $base->addChild($add->getName());

		foreach ($add->children() as $child)
			$this->mergeXML($new, $child);

		// merge attributes
		$this->mergeXMLattr($new, $add);

		// merge data
		foreach ($add as $key => $val)
			$new->$key = $val;
	}

	private function mergeXMLattr(&$base, $add)
	{
		foreach ($add->attributes() as $a => $b)
			$base->addAttribute($a, $b);
	}
}
