<?php

class Sync_Common extends Sync
{
	public function update() {}

	protected function execute($name_prefix)
	{
		global $options;

		$opts = getCmdOptions();
		$path = $opts->get_path();

		$script_name = $name_prefix . "_" . time();
		$dst_script = $path . "/" . $script_name . ".sh";

		$action = Action::factory();
		$action->upload($options['tmpdir'] . "/" . $name_prefix . ".sh", $dst_script, 0755);
		$action->exec("/bin/sh " . $dst_script);
		$action->exec("rm -f " . $dst_script);
	}

	// gets array(array('param' => '', 'descr' => ''));
	// create the executable script to update repository
	protected function create($script_name, $params = array())
	{
		global $options;

		$script = $options['tmpdir'] . "/" . $script_name . ".sh";
		if (!($fd = fopen($script, "w")))
			throw new Mirror_Exception("Unable to open file for writing: " . $script);

		if (!fwrite($fd, "#!/bin/sh\n\n")) {
			fclose($fd);
			throw new Mirror_Exception("Unable to write data into the file: " . $script);
		}

		$copy_target_code = '
copy_target()
{
	local src="$1"
	local dst="$2"
	' . $options['rsync_cmd'] . ' "$src" "$dst"
	local name="$(basename "$src")"
	if [ -L "$dst/$name" ]; then
		local link="$(readlink "$dst/$name")"
		[ "$link" != "$name" ] || return 0
		copy_target "$(dirname "$src")/$link" "$dst/$(dirname "$link")/"
	fi
}
';
		if (!fwrite($fd, $copy_target_code . "\n")) {
			fclose($fd);
			throw new Mirror_Exception("Unable to write data into the file: " . $script);
		}

		foreach ($params AS $param) {
			$code = "# " . $param['descr'] . "\n"
			      . "echo '" . $param['descr'] . "'\n"
			      . $param['param'] . "\n\n";
			if (!fwrite($fd, $code)) {
				fclose($fd);
				throw new Mirror_Exception("Unable to write data into the file: " . $script);
			}
		}

		fclose($fd);
	}

	protected function get_cmds(Product_Plesk $product, Config $config, $path)
	{
		global $options;

		// Gathering info for Plesk product
		$data = $product->get_info($config);
		$cmds = $product->get_cmd($data, $options['repos'], $path);

		// Gathering info for addons
		$addons = Product::factory(Types::ADDONS);
		$info = $addons->get_info($config);
		if ($info) {
			$data = array_merge($data, $info);
			$cmds = array_merge($cmds, $addons->get_cmd($data, $options['repos'], $path));
		}

		// Gathering info for essentials
		// should be last because of get_cmd uses
		// a common info from all modules
		$common = new Product_Common();
		$info = $common->get_info($config);
		if ($info) {
			$data = array_merge($data, $common->get_info($config));
			$cmds = array_merge($cmds, $common->get_cmd($data, $options['repos'], $path));
		}

		return $cmds;
	}
}
